// 
// Demo IWEB 2016-2017
//
// Controlador de puntuaciones
//

var models = require('../models');
var Sequelize = require('sequelize');

// Autoload el score asociado a :scoreId.
exports.load = function(req, res, next, scoreId) {
	models.Score.findById(scoreId)
    .then(function(score) {
  		if (score) {
    		req.score = score;
    		next();
  		} else { 
  			throw new Error('No existe el scoreId =' + scoreId);
  		}
    })
    .catch(function(error) { next(error); });
};

//---------------------------------------------

// Resuelve las llamadas a /scores y /scores/top3.
function indexOrTop3(req, res, next, top3) {

    var format = req.params.format || 'html';
    format = format.toLowerCase();

    var options = {};
    if (top3) {
        options.limit = 3;
        options.order = [['total','DESC']];
    } else {
        options.order = [['updatedAt','DESC']];
    }

    models.Score.findAll(options)
    .then(function(scores) { 
        switch (format) {
            case 'html':
                res.render('scores/index.ejs', {scores: scores});
                break;

            case 'json':
                res.send(scores);
                break;

            default:
                console.log('No se soporta el formato \".'+format+'\".');
                res.send(406);
        }
    })
    .catch(function(error) {
        switch (format) {
            case 'html':
                next(error);
                break;

            case 'json':
                console.log(error);
                res.send(500);
                break;

            default:
                console.log('No se soporta el formato \".'+format+'\".');
                res.send(406);
        }
    });
};


// GET /scores.format?
//
// Listado de todas las puntuaciones.
exports.index = function(req, res, next) {
    indexOrTop3(req, res, next, false);
};


// GET /scores/top3.format?
//
// Listado de las tres mejores puntuaciones.
exports.top3 = function(req, res, next) {

    indexOrTop3(req, res, next, true);
};


//---------------------------------------------

// GET /scores/:scoreId.:format?
//
// Muestra una puntuacion.
exports.show = function(req, res, next) {

    var format = req.params.format || 'html';
    format = format.toLowerCase();

    switch (format) {
        case 'html':
            res.render('scores/show', {score: req.score});
            break;

        case 'json':
            res.send(req.score);
            break;

        default:
            console.log('No se soporta el formato \".'+format+'\".');
            res.send(406);
    }
};


//---------------------------------------------


// GET /scores/new
//
// Formulario para crear un puntuacion nueva.
exports.new = function(req, res, next) {

    var score = models.Score.build({name: "", total: 0});
    res.render('scores/new', {score: score});
};


// POST /scores/create
//
// Crea una puntuacion nueva.
exports.create = function(req, res, next) {

    var score = { name:  req.body.name, 
                  total: Number(req.body.total) };

    // Guarda en la tabla Scores el nuevo score.
    models.Score.create(score)
    .then(function(score) {
        console.log('Score guardado con éxito.');   

        res.redirect("/scores");
    })
    .catch(Sequelize.ValidationError, function(error) {
        console.error('Errores en el formulario:');
        for (var i in error.errors) {
            console.error(error.errors[i].value);
        };
  
        res.render('scores/new', {score: score});
    })
    .catch(function(error) {
        console.error('Error al crear un Score: ' + error.message);
        next(error);
    }); 
};


//---------------------------------------------


// GET /scores/:scoreId/edit
//
// Formulario para editar una puntuacion.
exports.edit = function(req, res, next) {

    var score = req.score;  // autoload

    res.render('scores/edit', {score: score});
};


// PUT /scores/:scoreId
//
// Actualiza una puntuacion.
exports.update = function(req, res, next) {

    req.score.name  = req.body.name || req.score.name;
    req.score.total = Number(req.body.total) || req.score.total;

    req.score.save({fields: ["name", "total"]})
    .then(function(score) {

        console.log('Score editado con éxito.');  

        res.redirect("/scores");
    })
    .catch(Sequelize.ValidationError, function(error) {

        console.error('Errores en el formulario:');
        for (var i in error.errors) {
            console.error(error.errors[i].value);
        };

        res.render('scores/edit', {score: req.score});
    })
    .catch(function(error) {
        console.error('Error al editar el Score: ' + error.message);
        next(error);
    });
};


//---------------------------------------------


// DELETE /scores/:scoreId
//
// Borra una puntuacion.
exports.destroy = function(req, res, next) {

    req.score.destroy()
    .then( function() {
        console.log('Score borrado con éxito.');
        res.redirect("/scores");
    })
    .catch(function(error){
        console.error('Error al borrar el Score: ' + error.message);
        next(error);
    });
};


//---------------------------------------------


// DELETE /scores
//
// Borra todas las puntuaciones.
exports.clean = function(req, res, next) {

    models.Score.destroy({truncate: true})
    .then( function() {
        console.log('Borrados todos los scores con éxito.');
        res.redirect("/scores");
    })
    .catch(function(error){
        console.error('Error al borrar todos los scores: ' + error.message);
        next(error);
    });
};


//---------------------------------------------

